# pylint: disable=unsubscriptable-object
import itertools
from typing import List, Optional

from pydantic import BaseModel, root_validator, validator
from pydantic.fields import PrivateAttr

from aurorae.cnab240.v10_7 import models, types


class Company(BaseModel):
    bank_code: types.BankConventionCode
    registration_type: types.RegistrationType
    registration_number: types.RegistrationNumber
    bank_agency: types.BankAgencyNumber
    bank_agency_digit: Optional[types.BankDigitCheck] = None
    bank_account_number: types.BankAccountNumber
    bank_account_digit: Optional[types.BankDigitCheck] = None
    bank_account_agency_digit: Optional[types.BankDigitCheck] = None
    company_name: types.CompanyName
    bank_name: types.BankName
    address_location: types.NameAddress
    address_number: types.AddressNumber
    address_complement: types.AddressDetails
    address_city: types.AddressCityName
    address_cep: types.AddressCEP
    address_cep_complement: types.AddressCEPComplement
    address_state: types.AddressState

    class Config:
        validate_all = True
        validate_assignment = True

    @validator("bank_agency_digit", "bank_account_digit", "bank_account_agency_digit")
    def prevent_none(cls, v):  # noqa
        if v is None:
            return ""

        return v


class Employee(BaseModel):
    bank_code: types.BankCode
    registration_type: types.RegistrationType
    registration_number: types.RegistrationNumber
    bank_agency: types.BankAgencyNumber
    bank_agency_digit: Optional[types.BankDigitCheck] = None
    bank_account_number: types.BankAccountNumber
    bank_account_digit: Optional[types.BankDigitCheck] = None
    bank_account_agency_digit: Optional[types.BankDigitCheck] = None
    name: types.RecipientName
    initiation_form: Optional[types.InitiationForm]

    # Information 10
    address_location: Optional[types.NameAddress]
    # OR
    pix_tx_id: Optional[types.PIXTXID]

    # Information 11
    pix_key: Optional[types.PIXKey]
    # OR
    message: Optional[types.Message]
    # OR
    address_number: Optional[types.AddressNumber]
    address_complement: Optional[types.AddressDetails]
    address_district: Optional[types.AddressDistrict]
    address_city: Optional[types.SmallAddressCityName]
    address_cep: Optional[types.AddressCEP]
    address_cep_complement: Optional[types.AddressCEPComplement]
    address_state: Optional[types.AddressState]

    class Config:
        validate_all = True
        validate_assignment = True

    @validator("bank_agency_digit", "bank_account_digit", "bank_account_agency_digit")
    def prevent_none(cls, v):  # noqa
        if v is None:
            return ""

        return v

    @classmethod
    def has_all_address_fields(cls, values):  # noqa
        return (
            "address_number" in values
            and "address_complement" in values
            and "address_district" in values
            and "address_city" in values
            and "address_cep" in values
            and "address_cep_complement" in values
            and "address_state" in values
        )

    @classmethod
    def has_all_bank_info(cls, values):  # noqa
        return (
            "bank_code" in values
            and "bank_agency" in values
            and "bank_agency_digit" in values
            and "bank_account_number" in values
            and "bank_account_digit" in values
            and "bank_account_agency_digit" in values
        )

    @root_validator(pre=True)
    def check_information_10(cls, values):  # noqa
        """
        Field defined in CNAB document that requires information from multiple fields <field_09_3B>
        """
        assert (
            "address_location" in values or "pix_tx_id" in values
        ), "You must include the address_location or pix_tx_id"
        return values

    @root_validator(pre=True)
    def check_information_11(cls, values):  # noqa
        """
        Field defined in CNAB document that requires information from multiple fields <field_10_3B>
        """
        assert (
            "pix_key" in values
            or "message" in values
            or cls.has_all_address_fields(values)
        ), "You must include the pix_key or message or all address information"
        return values

    @root_validator(pre=True)
    def check_pix_info(cls, values):  # noqa
        if "pix_tx_id" in values:
            assert (
                "pix_tx_id" in values and "pix_key" in values
            ), "You must include all pix info, please add pix_key and pix_tx_id"

        return values

    @root_validator(pre=True)
    def check_address_fields(cls, values):  # noqa
        if "pix_tx_id" not in values and "pix_key" not in values:
            assert "address_location" in values and (
                cls.has_all_address_fields(values)
            ), "You must include all address information"

        return values

    @root_validator(pre=True)
    def check_initiation_form(cls, values):  # noqa
        if "initiation_form" not in values:
            values["initiation_form"] = types.InitiationFormEnum.bank_info

        init_form = values["initiation_form"]
        is_pix = types.InitiationFormEnum.bank_info != init_form
        if is_pix:
            has_pix_info = "pix_tx_id" in values and "pix_key" in values
            assert (
                is_pix and has_pix_info
            ), "Initiation form set to pix, please fill the pix information: pix_tx_id and pix_key"
            return values

        has_bank_info = cls.has_all_bank_info(values)
        assert has_bank_info, (
            "Initiation form set to bank, please fill the information: bank_code, bank_agency, "
            "bank_agency_digit, bank_account_number, bank_account_digit, bank_account_agency_digit"
        )
        return values


class Payment(BaseModel):
    employee: Optional[Employee]
    company: Optional[Company]
    employee_name: types.PersonName
    payment_amount: types.PaymentAmount
    payment_date: types.CNABDate
    release_method: Optional[types.ReleaseMethod]
    payment_method: Optional[types.PaymentMethod]
    ted_finality_code: Optional[types.TEDFinalityCode]
    doc_service_type: Optional[types.ServiceTypeComplement]

    # Information 12 Employee.pix_key OR
    # payment_date
    # payment_amount
    # and the following fields
    rebate_amount: types.RebateAmount = 0
    discount_amount: types.DiscountAmount = 0
    arrears_amount: types.ArrearsAmount = 0
    fine_amount: types.FineAmount = 0
    notify_recipient: types.NotifyRecipient = types.NotifyRecipientEnum.no_notification
    registration_number: types.InformationRegistrationType = str(
        types.CNABRegistrationTypeEnum.cpf.value
    )

    class Config:
        validate_all = True
        validate_assignment = True

    @root_validator(pre=True)
    def check_information_12(cls, values):  # noqa
        """
        Field defined in CNAB document that requires information from multiple fields <field_11_3B>
        """
        if "rebate_amount" in values:
            assert (
                "payment_date" in values
                or "payment_amount" in values
                or "rebate_amount" in values
                or "discount_amount" in values
                or "arrears_amount" in values
                or "fine_amount" in values
                or "notify_recipient" in values
                or "registration_number" in values
            ), (
                "You must include all fields: payment_date, payment_amount, rebate_amount, "
                "discount_amount, arrears_amount, fine_amount, notify_recipient, "
                "registration_number"
            )

        return values


class Payroll(BaseModel):
    company: Company
    employees: List[Employee]
    payments: List[Payment]

    _line_counter: int = PrivateAttr()

    class Config:
        validate_all = True
        validate_assignment = True

    @root_validator
    def check_payments_employees_exist(cls, values):  # noqa
        employees_names = map(
            lambda employee: employee.name, values.get("employees", [])
        )

        for payment in values.get("payments", []):
            assert (
                payment.employee_name in employees_names
            ), f"Employee not found for name: {payment.employee_name.value}"

        return values

    def _get_employee_by_name(self, employee_name):
        for employee in self.employees:
            if employee.name != employee_name:
                continue
            return employee

    def _get_payment_amount_sum(self):
        return sum([payment.payment_amount.value for payment in self.payments])

    def _get_cnab_batch(self):
        batch_header = models.CNABBatchHeader(
            initial_data=self.company.dict(), line_number=next(self._line_counter)
        )

        batch_records = []
        record_counter = 0
        for payment in self.payments:
            payment.company = self.company
            payment.employee = self._get_employee_by_name(payment.employee_name)

            record_counter += 1
            segment_a = models.CNABBatchSegmentA(
                payment=payment,
                record_number=record_counter,
                line_number=next(self._line_counter),
            )

            record_counter += 1
            segment_b = models.CNABBatchSegmentB(
                payment=payment,
                record_number=record_counter,
                line_number=next(self._line_counter),
            )

            batch_records.append(
                models.CNABBatchRecord(segment_a=segment_a, segment_b=segment_b)
            )

        batch_trailer = models.CNABBatchTrailer(
            company=self.company,
            total_batch_lines=record_counter + 2,
            sum_payment_values=self._get_payment_amount_sum(),
            line_number=next(self._line_counter),
        )
        return models.CNABBatch(
            header=batch_header, records=batch_records, trailer=batch_trailer
        )

    def get_cnab_file(self):
        self._line_counter = itertools.count(1)
        batches = []

        file_header = models.CNABHeader(
            initial_data=self.company.dict(), line_number=next(self._line_counter)
        )

        batches.append(self._get_cnab_batch())

        last_line_number = next(self._line_counter)
        file_trailer = models.CNABTrailer(
            company=self.company,
            total_file_lines=last_line_number,
            line_number=last_line_number,
        )

        return models.CNABFile(
            header=file_header, batches=batches, trailer=file_trailer
        )



import os

from openpyxl import load_workbook

from aurorae.providers.exceptions import EmptyFile, InvalidFileSize, InvalidFileType


def is_spreadsheet_type_xlsx(filename):
    """
    List of file signatures: https://en.wikipedia.org/wiki/List_of_file_signatures
    """
    xlsx_sig = b"\x50\x4B\x03\04"

    with open(filename, "rb") as f:
        file_bytes = f.read(4)
        return file_bytes == xlsx_sig


def is_spreadsheet_empty(filename):
    workbook = load_workbook(filename=filename, read_only=True)
    return all(sheet["A1"].value is None for sheet in workbook)


def is_spreadsheet_too_big(filename):
    max_size = 5242880
    return os.path.getsize(filename) > max_size


def validate_spreadsheet(filename):
    if is_spreadsheet_too_big(filename):
        raise InvalidFileSize
    if not is_spreadsheet_type_xlsx(filename):
        raise InvalidFileType
    if is_spreadsheet_empty(filename):
        raise EmptyFile



# pylint: disable=unsubscriptable-object
from datetime import datetime
from decimal import Decimal
from enum import Enum, IntEnum
from typing import ClassVar, Union

from pydantic import BaseModel, conint, constr, validator


STR_FILL_VALUE = " "
INT_FILL_VALUE = "0"

REGISTRATION_TYPE_MAP = {
    "ISENTO/NAO INFORMADO": 0,
    "CPF": 1,
    "CGC/CNPJ": 2,
    "PIS/PASEP": 3,
    "OUTROS": 9,
}

# fmt: off
VALID_CHARACTERS = (
    "abcdefghijklmnopqrstuvxywz"
    "ABCDEFGHIJKLMNOPQRSTUVXYWZ"
    "0123456789"
    ". "
)
# fmt: on


class CNABComposedField(BaseModel):
    def get_field_names(self):
        return self.__fields__.keys()

    def get_fields(self):
        fields = []
        for field_name in self.get_field_names():
            field = getattr(self, field_name)
            fields.append(field)
        return fields

    def as_fixed_width(self):
        formatted_value = ""
        for field in self.get_fields():
            formatted_value = f"{formatted_value}{field.as_fixed_width()}"

        assert len(formatted_value) == self._max_str_length

        return formatted_value


class CNABString(BaseModel):
    @property
    def value(self):
        value = self.__root__
        if isinstance(value, (CNABString, CNABComposedField)):
            value = value.as_fixed_width()
        return value

    @validator("__root__", pre=True, check_fields=False, allow_reuse=True)
    def validate_string(cls, value):  # noqa
        if not isinstance(value, (CNABString, CNABComposedField)):
            assert all(c in VALID_CHARACTERS for c in value), "Invalid characters"

        return value

    def as_fixed_width(self):
        return self.value.ljust(self._max_str_length, STR_FILL_VALUE).upper()


class CNABEnum(BaseModel):
    @property
    def value(self):
        return self.__root__.value

    def as_fixed_width(self):
        return str(self.value)


class CNABEnumFillInt(BaseModel):
    def as_fixed_width(self):
        return str(self.__root__.value).rjust(self._max_str_length, INT_FILL_VALUE)


class CNABEnumAlphaFill(BaseModel):
    def as_fixed_width(self):
        return str(self.__root__.value).ljust(self._max_str_length, STR_FILL_VALUE)


class PositiveInt(BaseModel):
    @property
    def value(self):
        return self.__root__

    @validator("__root__", pre=True, check_fields=False, allow_reuse=True)
    def validate_int(cls, value):  # noqa
        """Prevent coersion from float/decimal to int, allow coersion from string"""
        if isinstance(value, (Decimal, float)):
            raise ValueError(
                "value cannot be a float or Decimal, should be an integer instead"
            )

        return value


class CNABPositiveInt(PositiveInt):
    def as_fixed_width(self):
        return str(self.__root__).rjust(self._max_str_length, INT_FILL_VALUE)


class CNABAlphaPositiveInt(PositiveInt):
    def as_fixed_width(self):
        return str(self.__root__).ljust(self._max_str_length, STR_FILL_VALUE)


class CNABDate(BaseModel):
    _min_str_length: ClassVar[int] = 8
    _max_str_length: ClassVar[int] = 8

    __root__: constr(min_length=_min_str_length, max_length=_max_str_length)

    @validator("__root__", allow_reuse=True)
    def validate_date(cls, value):  # noqa
        try:
            datetime.strptime(value, "%d%m%Y")
        except ValueError:
            raise ValueError("{value} is not valid")
        else:
            return value

    def as_fixed_width(self):
        return self.__root__


class CNABTime(BaseModel):
    _min_str_length: ClassVar[int] = 6
    _max_str_length: ClassVar[int] = 6

    __root__: constr(min_length=_min_str_length, max_length=_max_str_length)

    @validator("__root__", allow_reuse=True)
    def validate_time(cls, value):  # noqa
        try:
            datetime.strptime(value, "%H%M%S")
        except ValueError:
            raise ValueError("{value} is not valid")
        else:
            return value

    def as_fixed_width(self):
        return self.__root__


class BankCode(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 3
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 999

    __root__: conint(ge=_min_int, le=_max_int)


class ServiceBatchEnum(str, Enum):
    file_header = "0000"
    file_trailer = "9999"


class ServiceBatch(CNABEnum):
    __root__: ServiceBatchEnum


class SequentialServiceBatch(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 4
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 9998

    __root__: conint(ge=_min_int, le=_max_int)


class EntryTypeEnum(IntEnum):
    file_header = 0
    batch_header = 1
    batch_initial_records = 2
    details = 3
    batch_final_records = 4
    batch_trailer = 5
    file_trailer = 9


class EntryType(CNABEnum):
    __root__: EntryTypeEnum


class RegistrationTypeEnum(Enum):
    exempt = "ISENTO/NAO INFORMADO"
    cpf = "CPF"
    cnpj = "CGC/CNPJ"
    pis = "PIS/PASEP"
    others = "OUTROS"


class CNABRegistrationTypeEnum(IntEnum):
    exempt = 0
    cpf = 1
    cnpj = 2
    pis = 3
    others = 9


class RegistrationType(CNABEnum):
    __root__: RegistrationTypeEnum


class CNABRegistrationType(CNABEnum):
    __root__: CNABRegistrationTypeEnum

    @validator("__root__", pre=True, allow_reuse=True)
    def validate_registration_type(cls, root_value):  # noqa
        if isinstance(root_value, RegistrationTypeEnum):
            return REGISTRATION_TYPE_MAP[root_value.value]
        return root_value


class RegistrationNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 14
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 99999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class BankConventionCode(CNABAlphaPositiveInt):
    _max_str_length: ClassVar[int] = 20
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 99999999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class BankAgencyNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 5
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 99999

    __root__: conint(ge=_min_int, le=_max_int)


class BankAccountNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 12
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class BankDigitCheck(CNABAlphaPositiveInt):
    _max_str_length: ClassVar[int] = 1
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 9
    __root__: constr(max_length=_max_str_length)

    @validator("__root__")
    def valid_int(cls, v):  # noqa
        try:
            v_as_int = int(v)
        except ValueError:
            return ""

        if v_as_int > cls._max_int:
            raise ValueError(f"{v_as_int} is greater than the max value {cls._max_int}")

        if v_as_int < cls._min_int:
            raise ValueError(f"{v_as_int} is smaller than the min value {cls._min_int}")

        return v


class CompanyName(CNABString):
    _max_str_length: ClassVar[int] = 30
    __root__: constr(max_length=_max_str_length)


class BankName(CNABString):
    _max_str_length: ClassVar[int] = 30
    __root__: constr(max_length=_max_str_length)


class RecipientName(CNABString):
    _max_str_length: ClassVar[int] = 30
    __root__: constr(max_length=_max_str_length)


class RemmitanceReturnCodeEnum(IntEnum):
    shipping = 1
    returning = 2


class RemmitanceReturnCode(CNABEnum):
    __root__: RemmitanceReturnCodeEnum


class FileSequentialNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 6
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 999999

    __root__: conint(ge=_min_int, le=_max_int)


class FileRecordDensityEnum(str, Enum):
    d1600 = "1600"
    d6250 = "6250"


class FileRecordDensity(CNABEnumFillInt):
    _max_str_length: ClassVar[int] = 5
    __root__: FileRecordDensityEnum


class BankReservedField(CNABString):
    _max_str_length: ClassVar[int] = 20
    __root__: constr(max_length=_max_str_length)


class CompanyReservedField(CNABString):
    _max_str_length: ClassVar[int] = 20
    __root__: constr(max_length=_max_str_length)


class FEBRABAN1(CNABString):
    _max_str_length = 1
    __root__: constr(max_length=_max_str_length)


class FEBRABAN6(CNABString):
    _max_str_length = 6
    __root__: constr(max_length=_max_str_length)


class FEBRABAN3(CNABString):
    _max_str_length: ClassVar[int] = 3
    __root__: constr(max_length=_max_str_length)


class FEBRABAN9(CNABString):
    _max_str_length: ClassVar[int] = 9
    __root__: constr(max_length=_max_str_length)


class FEBRABAN10(CNABString):
    _max_str_length: ClassVar[int] = 10
    __root__: constr(max_length=_max_str_length)


class FEBRABAN165(CNABString):
    _max_str_length = 165
    __root__: constr(max_length=_max_str_length)


class FEBRABAN205(CNABString):
    _max_str_length = 205
    __root__: constr(max_length=_max_str_length)


class FEBRABAN29(CNABString):
    _max_str_length: ClassVar[int] = 29
    __root__: constr(max_length=_max_str_length)


class SmallAddressCityName(CNABString):
    _max_str_length = 15
    __root__: constr(max_length=_max_str_length)


class AddressDistrict(CNABString):
    _max_str_length = 15
    __root__: constr(max_length=_max_str_length)


class RebateAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class DiscountAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class ArrearsAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class FineAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class InformationRegistrationType(CNABString):
    _max_str_length: ClassVar[int] = 15
    __root__: constr(max_length=_max_str_length)


class OperationTypeEnum(str, Enum):
    credit_entry = "C"
    debit_entry = "D"
    conciliation_statement = "E"
    cash_management_statement = "G"
    information_securities_secured_from_bank = "I"
    shipping_file = "R"
    return_file = "T"


class OperationType(CNABEnum):
    __root__: OperationTypeEnum


class ServiceTypeEnum(str, Enum):
    charge = "01"
    electronic_payment_slip = "03"
    bank_reconciliation = "04"
    debts = "05"
    check_custody = "06"
    cash_management = "07"
    query_margin_nformation = "08"
    registration_of_consignment_rRetention = "09"
    dividend_payment = "10"
    consignment_maintenance = "11"
    consignment_of_installments = "12"
    consignment_disallowance_inss = "13"
    inquiry_taxes_payable = "14"
    supplier_payment = "20"
    payment_of_bills_taxes = "22"
    interoperability_payment_institution_accounts = "23"
    buy = "25"
    buy_rotary = "26"
    payer_claim = "29"
    salary_payment = "30"
    payment_fees = "32"
    payment_stipend = "33"
    payment_prebends = "34"
    vendor = "40"
    forward_seller = "41"
    payment_of_insured = "50"
    payment_of_transit_traveler_expenses = "60"
    authorized_payment = "70"
    accredited_payment = "75"
    remuneration_payment = "77"
    payment_of_authorized_representatives = "80"
    payment_benefits = "90"
    miscellaneous_payments = "98"


class ServiceType(CNABEnum):
    __root__: ServiceTypeEnum


class ReleaseMethodEnum(str, Enum):
    credit_in_checking_account = "01"
    administrative_payment_check = "02"
    doc_ted = "03"
    card_salary = "04"
    credit_in_account_savings = "05"
    op_disposition = "10"
    payment_of_accounts_and_tributos_with_bar_code = "11"
    tax_DARF_normal = "16"
    tax_GPS = "17"
    tax_DARF_simple = "18"
    tax_IPTU_city = "19"
    payment_with_authentication = "20"
    tax_DARJ = "21"
    tax_GARE_SP_ICMS = "22"
    tax_GARE_SP_DR = "23"
    tax_GARE_SP_ITCMD = "24"
    tax_IPVA = "25"
    tax_licenciamento = "26"
    tax_DPVAT = "27"
    settlement_of_securities_of_the_bank = "30"
    payment_of_securities_from_other_banks = "31"
    current_account_statement = "40"
    TED_other_entitlement = "41"
    TED_same_ownership = "43"
    TED_transfer_from_investment_account = "44"
    PIX = "45"
    PIX_qr_code = "47"
    debit_checking_account = "50"
    statement_for_cash_management = "70"
    deposit_judicial_in_account_current = "71"
    deposit_judicial_in_savings = "72"
    statement_of_investment_account = "73"
    payment_of_municipal_taxes_ISS_own_bank = "80"
    payment_of_municipal_taxes_ISS_other_bank = "81"


class ReleaseMethod(CNABEnum):
    __root__: ReleaseMethodEnum


class BatchLayoutVersionNumberEnum(str, Enum):
    v0_46 = "046"


class BatchLayoutVersionNumber(CNABEnum):
    __root__: BatchLayoutVersionNumberEnum


class FileLayoutVersionNumberEnum(str, Enum):
    v10_3 = "103"


class FileLayoutVersionNumber(CNABEnum):
    __root__: FileLayoutVersionNumberEnum


class Message(CNABString):
    _max_str_length = 40
    __root__: constr(max_length=_max_str_length)


class NameAddress(CNABString):
    _max_str_length = 30
    __root__: constr(max_length=_max_str_length)


class AddressNumber(CNABPositiveInt):
    _max_str_length = 5
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 99999

    __root__: conint(ge=_min_int, le=_max_int)


class DetailRecordSegmentTypeEnum(str, Enum):
    segment_a = "A"
    segment_b = "B"
    segment_c = "C"


class DetailRecordSegmentType(CNABEnum):
    __root__: DetailRecordSegmentTypeEnum


class PersonName(CNABString):
    _max_str_length: ClassVar[int] = 30
    __root__: constr(max_length=_max_str_length)


class RecordSequentialNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 5
    _min_int: ClassVar[int] = 1
    _max_int: ClassVar[int] = 99999
    __root__: conint(ge=_min_int, le=_max_int)


class InitiationFormEnum(str, Enum):
    pix_phone = "01"
    pix_email = "02"
    pix_cpf_cnpj = "03"
    random_key = "04"
    bank_info = "05"


class InitiationForm(CNABEnumAlphaFill):
    _max_str_length: ClassVar[int] = 3
    __root__: InitiationFormEnum


class SIAPE6(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 6
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999

    __root__: conint(ge=_min_int, le=_max_int)


class ISPBCode(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 8
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 99999999

    __root__: conint(ge=_min_int, le=_max_int)


class AddressDetails(CNABString):
    _max_str_length = 15
    __root__: constr(max_length=_max_str_length)


class AddressCityName(CNABString):
    _max_str_length = 20
    __root__: constr(max_length=_max_str_length)


class AddressCEP(CNABPositiveInt):
    _max_str_length = 5
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 99999

    __root__: conint(ge=_min_int, le=_max_int)


class AddressCEPComplement(CNABPositiveInt):
    _max_str_length = 3
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999

    __root__: conint(ge=_min_int, le=_max_int)


class AddressState(CNABString):
    _max_str_length = 2
    __root__: constr(max_length=_max_str_length)


class PaymentMethodEnum(str, Enum):
    direct_debit = "01"
    loan_financing_debit = "02"
    credit_card_debit = "03"


class PaymentMethod(CNABEnum):
    __root__: PaymentMethodEnum


class MovimentationTypeEnum(IntEnum):
    inclusion = 0
    consultation = 1
    suspension = 2
    chargeback = 3
    reactivation = 4
    update = 5
    liquidation = 7
    exclusion = 9


class MovimentationType(CNABEnum):
    __root__: MovimentationTypeEnum


class MovimentationInstructionCodeEnum(str, Enum):
    inclusion_of_cleared_detail_record = "00"
    inclusion_of_blocked_detail_record = "09"
    change_payment_from_cleared_to_blocked = "10"
    change_blocked_from_payment_to_cleared = "11"
    change_in_security_value = "17"
    payment_date_change = "19"
    direct_payment_to_supplier = "23"
    wallet_maintenance_do_not_pay = "25"
    wallet_withdrawal_do_not_pay = "27"
    reversal_by_return_from_central_clearing_house = "33"
    payer_claim = "40"
    delete_previously_added_detail_record = "99"


class MovimentationInstructionCode(CNABEnum):
    __root__: MovimentationInstructionCodeEnum


class CentralClearingHouseCodeEnum(str, Enum):
    TED = "018"
    DOC = "700"
    # Used when it's necessary to send TED using the ISPB code
    # of the Recipient Financial Institution. In this case, it's mandatory to fill
    # the field "ISPB Code" - field 26.3B,
    # of the Payment Segment, as described in Note P015.
    TED_ISPB = "988"
    PIX = "009"


class CentralClearingHouseCode(CNABEnum):
    __root__: CentralClearingHouseCodeEnum


class CurrencyEnum(str, Enum):
    national_treasury_bonus_tr = "BTN"
    real = "BRL"
    us_dollar = "USD"
    portuguese_shield = "PTE"
    french_franc = "FRF"
    swiss_franc = "CHF"
    japanese_yen = "JPY"
    general_price_index = "IGP"
    general_market_price_index = "IGM"
    pound_sterling = "GBP"
    italian_lira = "ITL"
    german_mark = "DEM"
    daily_referential_rate = "TRD"
    standard_capital_unit = "UPC"
    standard_financing_unit = "UPF"
    tax_reference_unit = "UFR"
    european_currenc = "XEU"


class Currency(CNABEnum):
    __root__: CurrencyEnum


class CurrencyAmount(CNABPositiveInt):
    _max_str_length = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class PaymentAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class BankOrCompanyIssuedDocNumber(CNABString):
    _max_str_length: ClassVar[int] = 20
    __root__: constr(max_length=_max_str_length)


class PaymentEffectiveAmount(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 15
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999

    __root__: conint(ge=_min_int, le=_max_int)


class ServiceTypeComplementEnum(str, Enum):
    account_credit = "01"
    payment_of_rent_condominium = "02"
    payment_of_duplicate_securities = "03"
    payment_of_dividends = "04"
    payment_of_school_tuition = "05"
    salary_payment = "06"
    payment_to_suppliers = "07"
    transactions_of_foreign_exchange_funds_stock_exchange = "08"
    transfer_of_collection_payment_of_taxes = "09"
    international_transfer_in_real = "10"
    doc_for_savings = "11"
    doc_for_judicial_deposit = "12"
    others = "13"
    stipend_payment = "16"
    remuneration_to_the_member = "17"
    pagamento_of_fees = "18"
    payment_of_prebend = "19"


class ServiceTypeComplement(CNABEnum):
    __root__: ServiceTypeComplementEnum


class TEDFinalityCode(CNABAlphaPositiveInt):
    _max_str_length: ClassVar[int] = 5
    __root__: constr(max_length=_max_str_length)


class PaymentFinalityComplement(CNABString):
    _max_str_length: ClassVar[int] = 2
    __root__: constr(max_length=_max_str_length)


class NotifyRecipientEnum(str, Enum):
    no_notification = 0
    notify_only_sender = 2
    notify_only_recipient = 5
    notify_sender_and_recipient = 6
    notify_recipient_and_2_copies_to_sender = 7


class NotifyRecipient(CNABEnum):
    __root__: NotifyRecipientEnum


class ReturnOccurrenceCodes(CNABString):
    _max_str_length = 10
    __root__: constr(max_length=_max_str_length)


class RecordsNumber(CNABPositiveInt):
    _max_str_length = 6
    _min_int = 0
    _max_int = 999999
    __root__: conint(ge=_min_int, le=_max_int)


class ValuesSum(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 18
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999999
    __root__: conint(ge=_min_int, le=_max_int)


class CurrencyAmountsSum(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 18
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999999999999999
    __root__: conint(ge=_min_int, le=_max_int)


class DebitNotificationNumber(CNABPositiveInt):
    _max_str_length: ClassVar[int] = 6
    _min_int: ClassVar[int] = 0
    _max_int: ClassVar[int] = 999999

    __root__: conint(ge=_min_int, le=_max_int)


class ComposedField103B(CNABComposedField):
    _max_str_length: ClassVar[int] = 60

    address_number: AddressNumber
    address_complement: AddressDetails
    address_district: AddressDistrict
    address_city: SmallAddressCityName
    address_cep: AddressCEP
    address_cep_complement: AddressCEPComplement
    address_state: AddressState


class ComposedField113B(CNABComposedField):
    _max_str_length: ClassVar[int] = 99

    payment_date: CNABDate
    payment_amount: PaymentAmount
    rebate_amount: RebateAmount
    discount_amount: DiscountAmount
    arrears_amount: ArrearsAmount
    fine_amount: FineAmount
    registration_number: InformationRegistrationType
    notify_recipient: NotifyRecipient


class PIXTXID(CNABString):
    _max_str_length: ClassVar[int] = 30
    __root__: constr(max_length=_max_str_length)


class PIXKey(CNABString):
    _max_str_length: ClassVar[int] = 60
    __root__: constr(max_length=_max_str_length)


class Information10(CNABString):
    _max_str_length: ClassVar[int] = 35
    __root__: Union[NameAddress, PIXTXID]


class Information11(CNABString):
    _max_str_length: ClassVar[int] = 60
    __root__: Union[PIXKey, Message, ComposedField103B]


class Information12(CNABString):
    _max_str_length: ClassVar[int] = 99
    __root__: Union[PIXKey, ComposedField113B]



from pydantic import BaseModel, ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.fields import PrivateAttr


class Line(BaseModel):
    _total_positions: int = PrivateAttr(default=240)
    _formatted_value: str = PrivateAttr(default="")
    _line_number: int = PrivateAttr()

    def __init__(self, initial_data, line_number):  # noqa
        self._line_number = line_number
        self.validate_mapping()
        super().__init__(**initial_data)

    def validate_mapping(self):
        mapping = self.get_field_mapping()
        if not mapping:
            return

        mapping_set = set(mapping.keys())
        fields_set = set(self.get_field_names())

        diff = mapping_set.difference(fields_set)
        if not diff:
            return

        raise (
            ValidationError(
                [
                    ErrorWrapper(
                        Exception(
                            f"Config mapping keys: {diff} do not match valid fields"
                        ),
                        loc="config_mapping",
                    )
                ],
                model=self.__class__,
            )
        )

    def get_field_mapping(self):
        config = getattr(self, "Config")

        if not config:
            return None

        return getattr(config, "_mapping", None)

    def get_field_names(self):
        return self.__fields__.keys()

    def get_fields(self):
        fields = []
        for field_name in self.get_field_names():
            field = getattr(self, field_name)
            fields.append(field)
        return fields

    def get_field_info(self, field_name):
        field = self.__fields__[field_name]
        return field.field_info

    def as_fixed_width(self):
        for field in self.get_fields():
            self._formatted_value = f"{self._formatted_value}{field.as_fixed_width()}"

        assert len(self._formatted_value) == self._total_positions

        return self._formatted_value

    def as_html(self):
        formatted_html = f"<span class='line-number'>{self._line_number}.</span>"
        for field_name in self.get_field_names():
            field = getattr(self, field_name)
            field_info = self.get_field_info(field_name)

            field_tooltip = (
                f"{field_info.extra['code']} - "
                f"{field_name} - "
                f"{field_info.description}\n"
            )
            field_representation = field.as_fixed_width().replace(" ", "_")

            field_html_representation = (
                f"<span id='{field_name}' "
                f"data-tooltip='{field_tooltip}'>"
                f"{field_representation}"
                f"</span>"
            )
            formatted_html = f"{formatted_html}{field_html_representation}"
        return f"<div class='{self.__class__.__name__}'>{formatted_html}</div>"



import datetime


def get_field_G016():
    """File creation date"""
    return datetime.datetime.now().strftime("%d%m%Y")


def get_field_G017():
    """File creation time"""
    return datetime.datetime.now().strftime("%H%M%S")


def get_field_G018():
    """File sequential number"""
    return "1"


def get_field_G002_sequential():
    """
    Sequential number to uniquely identify a service batch. Created and controlled
    by the person responsible for the magnetic generation of the data contained in the file.

    Fill in with '0001' for the first batch of the file. For others: batch number
    previous plus 1. The number cannot be repeated within the file.
    """
    return 1


def get_field_G049():
    """
    The number of batches in a file. This number is
    defaulted as for now we only support one batch per file.
    """
    return "1"



from aurorae.payroll.models import Payroll
from aurorae.providers.spreadsheet import validators
from aurorae.providers.spreadsheet.models import Spreadsheet
from aurorae.providers.spreadsheet.utils import get_spreadsheet_data


class SpreadsheetHandler:
    def __init__(self, input_filename):
        validators.validate_spreadsheet(input_filename)
        self.spreadsheet = self.get_spreadsheet(input_filename)
        self.payroll = self.get_payroll()

    def get_spreadsheet(self, input_filename):
        spreadsheet_data = get_spreadsheet_data(input_filename)
        return Spreadsheet.parse_obj(spreadsheet_data)

    def get_payroll(self):
        return Payroll.parse_obj(self.spreadsheet.dict())

    def get_cnab_file(self):
        return self.payroll.get_cnab_file()



class InvalidFileType(Exception):
    def __str__(self):
        return "The file is not an Excel document"


class InvalidFileSize(Exception):
    def __str__(self):
        return "The file is over 5 MB"


class EmptyFile(Exception):
    def __str__(self):
        return "The file does not have content"



# pylint: disable=unsubscriptable-object
from __future__ import annotations

import os
from typing import TYPE_CHECKING, List, Optional

from pydantic import Field as FieldSchema
from pydantic.main import BaseModel

import aurorae
from aurorae.cnab240.base import Line
from aurorae.cnab240.v10_7 import lambdas, types


if TYPE_CHECKING:
    from aurorae.payroll.models import Company, Employee, Payment


class BaseConfig:
    allow_population_by_field_name = True

    @classmethod
    def alias_generator(cls, field: str) -> str:
        if field not in cls._mapping:
            return field
        return cls._mapping[field]


class CNABHeader(Line):
    field_01_0: types.BankCode = FieldSchema(
        description="Código do Banco na Compensação", code="G001"
    )
    field_02_0: types.ServiceBatch = FieldSchema(
        default=types.ServiceBatchEnum.file_header,
        description="Lote de Serviço",
        code="G002",
    )
    field_03_0: types.EntryType = FieldSchema(
        default=types.EntryTypeEnum.file_header,
        description="Tipo de Registro",
        code="G003",
    )
    field_04_0: types.FEBRABAN9 = FieldSchema(
        default="", description="Uso Exclusivo FEBRABAN / CNAB", code="G004"
    )
    field_05_0: types.CNABRegistrationType = FieldSchema(
        description="Tipo de Inscrição da Empresa", code="G005"
    )
    field_06_0: types.RegistrationNumber = FieldSchema(
        description="Número de Inscrição da Empresa", code="G006"
    )
    field_07_0: types.BankConventionCode = FieldSchema(
        description="Código do Convênio no Banco", code="G007"
    )
    field_08_0: types.BankAgencyNumber = FieldSchema(
        description="Agência Mantenedora da Conta", code="G008"
    )
    field_09_0: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Agência", code="G009"
    )
    field_10_0: types.BankAccountNumber = FieldSchema(
        description="Número da Conta Corrente", code="G010"
    )
    field_11_0: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Conta", code="G011"
    )
    field_12_0: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Ag/Conta", code="G012"
    )
    field_13_0: types.CompanyName = FieldSchema(
        description="Nome da Empresa", code="G013"
    )
    field_14_0: types.BankName = FieldSchema(description="Nome do Banco", code="G014")
    field_15_0: types.FEBRABAN10 = FieldSchema(
        default="", description="Uso Exclusivo FEBRABAN / CNAB", code="G004"
    )
    field_16_0: types.RemmitanceReturnCode = FieldSchema(
        description="Código Remessa / Retorno",
        default=types.RemmitanceReturnCodeEnum.shipping,
        code="G015",
    )
    field_17_0: types.CNABDate = FieldSchema(
        description="Data de Geração do Arquivo",
        default_factory=lambdas.get_field_G016,
        code="G016",
    )
    field_18_0: types.CNABTime = FieldSchema(
        description="Hora de Geração do Arquivo",
        default_factory=lambdas.get_field_G017,
        code="G017",
    )
    field_19_0: types.FileSequentialNumber = FieldSchema(
        description="Número Seqüencial do Arquivo",
        code="G018",
        default_factory=lambdas.get_field_G018,
    )
    field_20_0: types.FileLayoutVersionNumber = FieldSchema(
        description="Número da Versão do Layout do Arquivo",
        code="G019",
        default=types.FileLayoutVersionNumberEnum.v10_3,
    )
    field_21_0: types.FileRecordDensity = FieldSchema(
        description="Densidade de Gravação do Arquivo",
        code="G020",
        default=types.FileRecordDensityEnum.d1600,
    )
    field_22_0: types.BankReservedField = FieldSchema(
        default="", description="Para Uso Reservado do Banco", code="G021"
    )
    field_23_0: types.CompanyReservedField = FieldSchema(
        default="", description="Para Uso Reservado da Empresa", code="G022"
    )
    field_24_0: types.FEBRABAN29 = FieldSchema(
        default="", description="Uso Exclusivo FEBRABAN / CNAB", code="G004"
    )

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True
        _mapping = {
            "field_01_0": "bank_code",
            "field_05_0": "registration_type",
            "field_06_0": "registration_number",
            "field_07_0": "bank_code",
            "field_08_0": "bank_agency",
            "field_09_0": "bank_agency_digit",
            "field_10_0": "bank_account_number",
            "field_11_0": "bank_account_digit",
            "field_12_0": "bank_account_agency_digit",
            "field_13_0": "company_name",
            "field_14_0": "bank_name",
        }


class CNABBatchHeader(Line):
    field_01_1: types.BankCode = FieldSchema(
        description="Código do Banco na Compensação",
        code="G001",
    )
    field_02_1: types.SequentialServiceBatch = FieldSchema(
        default_factory=lambdas.get_field_G002_sequential,
        description="Lote de Serviço",
        code="G002",
    )
    field_03_1: types.EntryType = FieldSchema(
        default=types.EntryTypeEnum.batch_header,
        description="Tipo de Registro",
        code="G003",
    )
    field_04_1: types.OperationType = FieldSchema(
        default=types.OperationTypeEnum.credit_entry,
        description="Tipo da Operação",
        code="G028",
    )
    field_05_1: types.ServiceType = FieldSchema(
        default=types.ServiceTypeEnum.salary_payment,
        description="Tipo do Serviço",
        code="G025",
    )
    field_06_1: types.ReleaseMethod = FieldSchema(
        default=types.ReleaseMethodEnum.credit_in_checking_account,
        description="Forma de Lançamento",
        code="G029",
    )
    field_07_1: types.BatchLayoutVersionNumber = FieldSchema(
        default=types.BatchLayoutVersionNumberEnum.v0_46,
        description="Nº da Versão do Layout do Lote",
        code="G030",
    )
    field_08_1: types.FEBRABAN1 = FieldSchema(
        default="",
        description="Uso Exclusivo da FEBRABAN/CNAB",
        code="G004",
    )
    field_09_1: types.CNABRegistrationType = FieldSchema(
        description="Tipo de Inscrição da Empresa",
        code="G005",
    )
    field_10_1: types.RegistrationNumber = FieldSchema(
        description="Número de Inscrição da Empresa",
        code="G006",
    )
    field_11_1: types.BankConventionCode = FieldSchema(
        description="Código do Convênio no Banco",
        code="G007",
    )
    field_12_1: types.BankAgencyNumber = FieldSchema(
        description="Agência Mantenedora da Conta",
        code="G008",
    )
    field_13_1: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Agência",
        code="G009",
    )
    field_14_1: types.BankAccountNumber = FieldSchema(
        description="Número da Conta Corrente",
        code="G010",
    )
    field_15_1: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Conta",
        code="G011",
    )
    field_16_1: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Ag/Conta",
        code="G012",
    )
    field_17_1: types.CompanyName = FieldSchema(
        description="Nome da Empresa",
        code="G013",
    )
    field_18_1: types.Message = FieldSchema(
        description="Mensagem",
        default="",
        code="G031",
    )
    field_19_1: types.NameAddress = FieldSchema(
        description="Nome da Rua, Av, Pça, Etc",
        default="",
        code="G032",
    )
    field_20_1: types.AddressNumber = FieldSchema(
        description="Número do Local",
        code="G032",
    )
    field_21_1: types.AddressDetails = FieldSchema(
        description="Casa, Apto, Sala, Etc",
        code="G032",
    )
    field_22_1: types.AddressCityName = FieldSchema(
        description="Nome da Cidade",
        code="G033",
    )
    field_23_1: types.AddressCEP = FieldSchema(
        description="CEP",
        code="G034",
    )
    field_24_1: types.AddressCEPComplement = FieldSchema(
        description="Complemento do CEP",
        code="G035",
    )
    field_25_1: types.AddressState = FieldSchema(
        description="Sigla do Estado",
        code="G036",
    )
    field_26_1: types.PaymentMethod = FieldSchema(
        default=types.PaymentMethodEnum.direct_debit,
        description="Indicativo da Forma de Pagamento do Serviço",
        code="P014",
    )
    field_27_1: types.FEBRABAN6 = FieldSchema(
        default="",
        description="Uso Exclusivo FEBRABAN/CNAB",
        code="G004",
    )
    field_28_1: types.ReturnOccurrenceCodes = FieldSchema(
        default="",
        description="Códigos das Ocorrências p/ Retorno",
        code="G059",
    )

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True
        _mapping = {
            "field_01_1": "bank_code",
            "field_09_1": "registration_type",
            "field_10_1": "registration_number",
            "field_11_1": "bank_code",
            "field_12_1": "bank_agency",
            "field_13_1": "bank_agency_digit",
            "field_14_1": "bank_account_number",
            "field_15_1": "bank_account_digit",
            "field_16_1": "bank_account_agency_digit",
            "field_17_1": "company_name",
            "field_19_1": "address_location",
            "field_20_1": "address_number",
            "field_21_1": "address_complement",
            "field_22_1": "address_city",
            "field_23_1": "address_cep",
            "field_24_1": "address_cep_complement",
            "field_25_1": "address_state",
        }


class CNABBatchSegmentA(Line):
    field_01_3A: types.BankCode = FieldSchema(
        description="Código do Banco na Compensação", code="G001"
    )
    field_02_3A: types.SequentialServiceBatch = FieldSchema(
        description="Lote de Serviço",
        code="G002",
        default_factory=lambdas.get_field_G002_sequential,
    )
    field_03_3A: types.EntryType = FieldSchema(
        description="Tipo de Registro", code="G003", default=types.EntryTypeEnum.details
    )
    field_04_3A: types.RecordSequentialNumber = FieldSchema(
        description="Nº Seqüencial do Registro no Lote",
        code="G038",
    )
    field_05_3A: types.DetailRecordSegmentType = FieldSchema(
        description="Código de Segmento do Reg. Detalhe",
        code="G039",
        default=types.DetailRecordSegmentTypeEnum.segment_a,
    )
    field_06_3A: types.MovimentationType = FieldSchema(
        description="Tipo de Movimento",
        code="G060",
        default=types.MovimentationTypeEnum.inclusion,
    )
    field_07_3A: types.MovimentationInstructionCode = FieldSchema(
        description="Código da Instrução p/ Movimento",
        code="G061",
        default=types.MovimentationInstructionCodeEnum.inclusion_of_cleared_detail_record,
    )
    field_08_3A: types.CentralClearingHouseCode = FieldSchema(
        description="Código da Câmara Centralizadora",
        code="P001",
        default=types.CentralClearingHouseCodeEnum.TED,
    )
    field_09_3A: types.BankCode = FieldSchema(
        description="Código do Banco do Favorecido", code="P002"
    )
    field_10_3A: types.BankAgencyNumber = FieldSchema(
        description="Ag. Mantenedora da Cta do Favor.", code="G008"
    )
    field_11_3A: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Agência", code="G009"
    )
    field_12_3A: types.BankAccountNumber = FieldSchema(
        description="Número da Conta Corrente", code="G010"
    )
    field_13_3A: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da Conta", code="G011"
    )
    field_14_3A: types.BankDigitCheck = FieldSchema(
        description="Dígito Verificador da AG/Conta", code="G012"
    )
    field_15_3A: types.PersonName = FieldSchema(
        description="Nome do Favorecido", code="G013"
    )
    field_16_3A: types.BankOrCompanyIssuedDocNumber = FieldSchema(
        description="Nº do Docum. Atribuído p/ Empresa", code="G064", default=""
    )
    field_17_3A: types.CNABDate = FieldSchema(
        description="Data do Pagamento", code="P009"
    )
    field_18_3A: types.Currency = FieldSchema(
        description="Tipo da Moeda", code="G040", default=types.CurrencyEnum.real
    )
    field_19_3A: types.CurrencyAmount = FieldSchema(
        description="Quantidade da Moeda", code="G041", default=0
    )
    field_20_3A: types.PaymentAmount = FieldSchema(
        description="Valor do Pagamento", code="P010"
    )
    field_21_3A: types.BankOrCompanyIssuedDocNumber = FieldSchema(
        description="Nº do Docum. Atribuído pelo Banco", code="G043", default=""
    )
    field_22_3A: types.CNABDate = FieldSchema(
        description="Data Real da Efetivação Pagto", code="P003"
    )
    field_23_3A: types.PaymentEffectiveAmount = FieldSchema(
        description="Valor Real da Efetivação do Pagto", code="P004", default=0
    )
    field_24_3A: types.Message = FieldSchema(
        description="Outras Informações", code="G031", default=""
    )
    field_25_3A: types.ServiceTypeComplement = FieldSchema(
        description="Compl. Tipo Serviço",
        code="P005",
        default=types.ServiceTypeComplementEnum.account_credit,
    )
    field_26_3A: types.TEDFinalityCode = FieldSchema(
        description="Codigo finalidade da TED", code="P011", default=""
    )
    field_27_3A: types.PaymentFinalityComplement = FieldSchema(
        description="Complemento de finalidade pagto.", code="P013", default=""
    )
    field_28_3A: types.FEBRABAN3 = FieldSchema(
        description="Uso Exclusivo FEBRABAN/CNAB", code="G004", default=""
    )
    field_29_3A: types.NotifyRecipient = FieldSchema(
        description="Aviso ao Favorecido",
        code="P006",
        default=types.NotifyRecipientEnum.no_notification,
    )
    field_30_3A: types.ReturnOccurrenceCodes = FieldSchema(
        description="Códigos das Ocorrências para Retorno", code="G059", default=""
    )

    _nested_mapping = {
        "field_01_3A": "company.bank_code",
        "field_09_3A": "employee.bank_code",
        "field_10_3A": "employee.bank_agency",
        "field_11_3A": "employee.bank_agency_digit",
        "field_12_3A": "employee.bank_account_number",
        "field_13_3A": "employee.bank_account_digit",
        "field_14_3A": "employee.bank_account_agency_digit",
        "field_15_3A": "employee.name",
        "field_17_3A": "payment.payment_date",
        "field_20_3A": "payment.payment_amount",
        "field_22_3A": "payment.payment_date",
    }

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True
        _mapping = {"field_04_3A": "record_number"}

    def __init__(self, payment: Payment, record_number: int, line_number: int) -> None:
        employee = payment.employee
        company = payment.company

        initial_data = {
            "payment": payment.dict(),
            "company": company.dict(),
            "employee": employee.dict(),
        }

        data = self._map_nested_values(initial_data)
        data["record_number"] = record_number
        super().__init__(data, line_number=line_number)

    def _map_nested_values(self, initial_data: dict) -> None:
        data = {}
        for key, nested_path in self._nested_mapping.items():
            [entity_key, field_key] = nested_path.split(".")

            entity = initial_data[entity_key]
            data[key] = entity[field_key]

        return data


class CNABBatchSegmentB(Line):
    field_01_3B: types.BankCode = FieldSchema(
        name="Código do Banco na Compensação",
        code="G001",
    )
    field_02_3B: types.SequentialServiceBatch = FieldSchema(
        default_factory=lambdas.get_field_G002_sequential,
        name="Lote de Serviço",
        code="G002",
    )
    field_03_3B: types.EntryType = FieldSchema(
        default=types.EntryTypeEnum.details,
        name="Tipo de Registro",
        code="G003",
    )
    field_04_3B: types.RecordSequentialNumber = FieldSchema(
        name="Nº Seqüencial do Registro no Lote",
        code="G038",
    )
    field_05_3B: types.DetailRecordSegmentType = FieldSchema(
        default=types.DetailRecordSegmentTypeEnum.segment_b,
        name="Código de Segmento do Reg. Detalhe",
        code="G039",
    )
    field_06_3B: types.InitiationForm = FieldSchema(
        default=types.InitiationFormEnum.bank_info,
        name="* Forma de Iniciação",
        code="G100",
    )
    field_07_3B: types.CNABRegistrationType = FieldSchema(
        name="Tipo de Inscrição do Favorecido",
        code="G005",
    )
    field_08_3B: types.RegistrationNumber = FieldSchema(
        name="Nº de Inscrição do Favorecido",
        code="G006",
    )
    field_09_3B: types.Information10 = FieldSchema(
        default="",
        name="Informação 10",
        code="G101",
    )
    field_10_3B: types.Information11 = FieldSchema(
        default="",
        name="Informação 11",
        code="G101",
    )
    field_11_3B: types.Information12 = FieldSchema(
        default="",
        name="Informação 12",
        code="G101",
    )
    field_12_3B: types.SIAPE6 = FieldSchema(
        default=0,
        name="Uso Exclusivo para o SIAPE",
        code="P012",
    )
    field_13_3B: types.ISPBCode = FieldSchema(
        default=0,
        name="Código ISPB",
        code="P015",
    )

    def get_information_10(self, employee: Employee):
        return types.Information10.parse_obj(employee.address_location.value)

    def get_information_11(self, employee: Employee):
        return types.ComposedField103B.parse_obj(employee)

    def get_information_12(self, payment: Payment):
        return types.ComposedField113B.parse_obj(payment)

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True

        _mapping = {
            "field_01_3B": "bank_code",
            "field_04_3B": "record_number",
            "field_07_3B": "registration_type",
            "field_08_3B": "registration_number",
            "field_09_3B": "information_10",
            "field_10_3B": "information_11",
            "field_11_3B": "information_12",
        }

    def __init__(self, payment: Payment, record_number: int, line_number: int) -> None:
        employee = payment.employee
        initial_data = employee.dict()
        initial_data["record_number"] = record_number
        initial_data["information_10"] = self.get_information_10(employee)
        initial_data["information_11"] = self.get_information_11(employee)
        initial_data["information_12"] = self.get_information_12(payment)
        super().__init__(initial_data, line_number)


class CNABBatchSegmentC(Line):
    pass


class CNABBatchTrailer(Line):
    field_01_5: types.BankCode = FieldSchema(
        description="Código do Banco na Compensação", code="G001"
    )
    field_02_5: types.SequentialServiceBatch = FieldSchema(
        description="Lote de Serviço",
        code="G002",
        default_factory=lambdas.get_field_G002_sequential,
    )
    field_03_5: types.EntryType = FieldSchema(
        description="Tipo de Registro",
        code="G003",
        default=types.EntryTypeEnum.batch_trailer,
    )
    field_04_5: types.FEBRABAN9 = FieldSchema(
        description="Uso Exclusivo FEBRABAN/CNAB", code="G004", default=""
    )
    field_05_5: types.RecordsNumber = FieldSchema(
        description="Quantidade de Registros do Lote", code="G057"
    )
    field_06_5: types.ValuesSum = FieldSchema(
        description="Somatória dos Valores", code="P007"
    )
    field_07_5: types.CurrencyAmountsSum = FieldSchema(
        description="Somatória de Quantidade de Moedas", code="G058", default=0
    )
    field_08_5: types.DebitNotificationNumber = FieldSchema(
        description="Número Aviso de Débito ", code="G066", default=0
    )
    field_09_5: types.FEBRABAN165 = FieldSchema(
        description="Uso Exclusivo FEBRABAN/CNAB", code="G004", default=""
    )
    field_10_5: types.ReturnOccurrenceCodes = FieldSchema(
        description="Códigos das Ocorrências para Retorno", code="G059", default=""
    )

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True
        _mapping = {
            "field_01_5": "bank_code",
            "field_05_5": "total_batch_lines",
            "field_06_5": "payment_amount_sum",
        }

    def __init__(
        self,
        company: Company,
        sum_payment_values: str,
        total_batch_lines: int,
        line_number,
    ):

        initial_data = company.dict()
        initial_data["total_batch_lines"] = total_batch_lines
        initial_data["payment_amount_sum"] = sum_payment_values
        super().__init__(initial_data, line_number)


class CNABBatchRecord(BaseModel):
    segment_a: CNABBatchSegmentA
    segment_b: CNABBatchSegmentB
    segment_c: Optional[CNABBatchSegmentC]

    def as_fixed_width(self):
        fixed_width_str = (
            f"{self.segment_a.as_fixed_width()}\n"
            f"{self.segment_b.as_fixed_width()}\n"
        )
        return fixed_width_str

    def as_html(self):
        html_result = f"{self.segment_a.as_html()}{self.segment_b.as_html()}"
        return html_result


class CNABBatch(BaseModel):
    header: CNABBatchHeader
    records: List[CNABBatchRecord]
    trailer: CNABBatchTrailer

    def as_fixed_width(self):
        fixed_width_str = ""

        fixed_width_str += f"{self.header.as_fixed_width()}\n"
        for record in self.records:
            fixed_width_str += f"{record.as_fixed_width()}"
        fixed_width_str += f"{self.trailer.as_fixed_width()}\n"

        return fixed_width_str

    def as_html(self):
        html_result = ""

        html_result += f"{self.header.as_html()}"
        for record in self.records:
            html_result += f"{record.as_html()}"
        html_result += f"{self.trailer.as_html()}"

        return html_result


class CNABTrailer(Line):
    field_01_9: types.BankCode = FieldSchema(
        description="Código do Banco na Compensação", code="G001"
    )
    field_02_9: types.ServiceBatch = FieldSchema(
        default=types.ServiceBatchEnum.file_trailer,
        description="Lote de Serviço",
        code="G002",
    )
    field_03_9: types.EntryType = FieldSchema(
        default=types.EntryTypeEnum.file_trailer,
        description="Tipo de Registro",
        code="G003",
    )
    field_04_9: types.FEBRABAN9 = FieldSchema(
        default="", description="Uso Exclusivo FEBRABAN/CNAB", code="G004"
    )
    field_05_9: types.RecordsNumber = FieldSchema(
        description="Quantidade de Lotes do Arquivo",
        code="G049",
        default_factory=lambdas.get_field_G049,
    )
    field_06_9: types.RecordsNumber = FieldSchema(
        description="Quantidade de Registros do Arquivo", code="G056"
    )
    field_07_9: types.RecordsNumber = FieldSchema(
        default=0, description="Qtde de Contas p/ Conc. (Lotes)", code="G037"
    )
    field_08_9: types.FEBRABAN205 = FieldSchema(
        default="", description="Uso Exclusivo FEBRABAN/CNAB", code="G004"
    )

    class Config(BaseConfig):
        validate_all = True
        validate_assignment = True
        _mapping = {"field_01_9": "bank_code", "field_06_9": "total_file_lines"}

    def __init__(self, company: Company, total_file_lines: int, line_number: int):
        initial_data = company.dict()
        initial_data["total_file_lines"] = total_file_lines
        super().__init__(initial_data, line_number)


class CNABFile(BaseModel):
    header: CNABHeader
    batches: List[CNABBatch]
    trailer: CNABTrailer

    def as_fixed_width(self):
        fixed_width_str = ""

        fixed_width_str += f"{self.header.as_fixed_width()}\n"
        for batch in self.batches:
            fixed_width_str += f"{batch.as_fixed_width()}"
        fixed_width_str += f"{self.trailer.as_fixed_width()}"

        return fixed_width_str

    def as_html(self):
        html_result = ""

        html_result += f"{self.header.as_html()}"
        for batch in self.batches:
            html_result += f"{batch.as_html()}"
        html_result += f"{self.trailer.as_html()}"

        return html_result

    def generate_file(self, output_filename):
        file_path = f"{output_filename}"

        with open(os.path.expanduser(file_path), "w") as f:
            file_content = self.as_fixed_width()
            f.write(file_content)

    def generate_html_file(self, output_filename):
        file_path = f"{output_filename.with_suffix('.html')}"

        root_path = os.path.dirname(aurorae.__file__)
        stylesheet_filename = os.path.join(root_path, "staticfiles/styles.css")

        with open(os.path.expanduser(file_path), "w") as f:
            html_content = self.as_html()
            f.write("<html><head>")
            f.write(f"<link href='{stylesheet_filename}' rel='stylesheet'>")
            f.write("</head><body>")
            f.write(html_content)
            f.write("</body></html>")



from fastapi import FastAPI, Response

from aurorae.payroll.models import Payroll
from aurorae.providers.spreadsheet.models import Spreadsheet


app = FastAPI()


@app.post("/parse_from_spreadsheet")
async def parse_from_spreadsheet_data(spreadsheet_data: Spreadsheet):
    spreadsheet = Spreadsheet.parse_obj(spreadsheet_data)
    payroll = Payroll.parse_obj(spreadsheet.dict())

    cnab_file = payroll.get_cnab_file()
    response = cnab_file.as_fixed_width()

    return Response(content=response, media_type="text/html")



import os
from pathlib import Path

import aurorae
from aurorae.providers.spreadsheet.handler import SpreadsheetHandler
from aurorae.providers.utils import parse_args


def generate_cnab_files():
    args = parse_args()

    handler = SpreadsheetHandler(input_filename=args.input_filename)
    cnab = handler.get_cnab_file()
    cnab.generate_file(output_filename=args.output_filename)
    cnab.generate_html_file(output_filename=args.output_filename)


def generate_cnab_sample():
    root_path = os.path.dirname(aurorae.__file__)

    output_filename = "aurorae_cnab.txt"
    input_filename = os.path.join(root_path, "sample/spreadsheet_sample.xlsx")

    handler = SpreadsheetHandler(input_filename=input_filename)
    cnab = handler.get_cnab_file()

    cnab.generate_file(output_filename=Path(output_filename))
    print(f"New CNAB file generated: {output_filename}")

    cnab.generate_html_file(output_filename=Path(output_filename))
    html_file = f"{Path(output_filename).with_suffix('.html')}"
    print(f"New CNAB debug file generated: {html_file}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aurorae',
 'aurorae.cnab240',
 'aurorae.cnab240.v10_7',
 'aurorae.payroll',
 'aurorae.providers',
 'aurorae.providers.api',
 'aurorae.providers.spreadsheet']

package_data = \
{'': ['*'], 'aurorae': ['sample/*', 'staticfiles/*']}

install_requires = \
['Pygments>=2.9.0,<3.0.0',
 'attrs>=21.2.0,<22.0.0',
 'backcall>=0.2.0,<0.3.0',
 'decorator>=5.0.9,<6.0.0',
 'et-xmlfile>=1.1.0,<2.0.0',
 'fastapi>=0.68.0,<0.69.0',
 'iniconfig>=1.1.1,<2.0.0',
 'ipython>=7.25.0,<8.0.0',
 'ipython_genutils>=0.2.0,<0.3.0',
 'jedi>=0.18.0,<0.19.0',
 'openpyxl>=3.0.7,<4.0.0',
 'packaging>=21.0,<22.0',
 'parso>=0.8.2,<0.9.0',
 'pexpect>=4.8.0,<5.0.0',
 'pickleshare>=0.7.5,<0.8.0',
 'pluggy>=0.13.1,<0.14.0',
 'prompt-toolkit>=3.0.19,<4.0.0',
 'ptyprocess>=0.7.0,<0.8.0',
 'py>=1.10.0,<2.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'pyparsing>=2.4.7,<3.0.0',
 'requests>=2.26.0,<3.0.0',
 'toml>=0.10.2,<0.11.0',
 'traitlets>=5.0.5,<6.0.0',
 'uvicorn[standard]>=0.15.0,<0.16.0',
 'wcwidth>=0.2.5,<0.3.0']

entry_points = \
{'console_scripts': ['generate_cnab = '
                     'aurorae.cnab240.writer:generate_cnab_files',
                     'generate_cnab_sample = '
                     'aurorae.cnab240.writer:generate_cnab_sample']}

setup_kwargs = {
    'name': 'aurorae',
    'version': '0.0.1',
    'description': 'A Python implementation of the CNAB240 file to perform bulk payments.',
    'long_description': '# aurorae\n\n[![CI](https://github.com/vintasoftware/aurorae/actions/workflows/actions.yaml/badge.svg)](https://github.com/vintasoftware/aurorae/actions/workflows/actions.yaml)\n[![Coverage Status](https://coveralls.io/repos/github/vintasoftware/aurorae/badge.svg?branch=main)](https://coveralls.io/github/vintasoftware/aurorae?branch=main)\n[![Documentation Status](https://readthedocs.org/projects/aurorae/badge/?version=latest)](https://aurorae.readthedocs.io/en/latest/?badge=latest)\n[![License: MIT](https://img.shields.io/github/license/vintasoftware/django-react-boilerplate.svg)](LICENSE.txt)\n\n**aurorae** is a tool to generate fixed-width CNAB240 files to perform bulk payments.\n\n### aurorae _does..._\n- Generates CNAB240 files for bulk payments\n- Allows easy extension of different types of input files\n\n### aurorae _does not..._\n- Address charge or as Brazilian banks call "cobrança"\n- Address payments by PIX, we only support payments through bank information\n\nBut, pull requests are welcomed.\n\n## How It Works\n**aurorae** uses Python type hinting for data validation and generation of fixed-width CNAB 240 files. The library receives as inputs an spreadsheet that must be a match of the Pydantic model [Spreadsheet](https://github.com/vintasoftware/aurorae/blob/main/aurorae/providers/spreadsheet/models.py), a general handler parses the initial data to an intermediary representation used by the CNAB240 module to generate files. Different types of inputs are supported by library through the creation of new providers, check the [spreadsheet provider](https://github.com/vintasoftware/aurorae/tree/main/aurorae/providers/spreadsheet) for an example.\n\nThe historic and architecture details can be found on the [project\'s ADRs](https://github.com/vintasoftware/aurorae/blob/main/docs/adr/README.md).\n\n## Requirements\n\n- Python (>3)\n- openpyxl (3.0.7)\n- pydantic (>1.8.2)\n\n## Usage\nTo run aurorae with test data:\n```bash\ngenerate_cnab_sample\n```\n\n## Security\nWe take aurorae\'s security and our users\' trust seriously, therefore we do not save any information (from payments or not) sent by users. If you believe you have found a security issue, please responsibly disclose by contacting: [flavio@vinta.com.br](flavio@vinta.com.br)\n\n## Releases\n\nSee [CHANGELOG.md](https://github.com/vintasoftware/aurorae/blob/main/CHANGELOG.md).\n\n## Credits\n\nThis project is maintained by [open-source contributors](https://github.com/vintasoftware/aurorae/blob/main/AUTHORS.rst) and [Vinta Software](https://www.vintasoftware.com/).\n\n## Commercial Support\n\n[Vinta Software](https://www.vintasoftware.com/) is always looking for exciting work, so if you need any commercial support, feel free to get in touch: contact@vinta.com.br\n',
    'author': 'Mariane Pastor (Vinta Software)',
    'author_email': 'mariane.pastor@vinta.com.br',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from openpyxl import load_workbook


def worksheet_dict_reader(worksheet):
    """
    A generator for the rows in a given worksheet. It maps columns on
    the first row of the spreadsheet to each of the following lines
    returning a dict like {
        "header_column_name_1": "value_column_1",
        "header_column_name_1": "value_column_2"
    }
    :param worksheet: a worksheet object
    :type worksheet: `openpyxl.worksheet.ReadOnlyWorksheet`
    """
    rows = worksheet.iter_rows(values_only=True)

    # pylint: disable=stop-iteration-return
    header = list(filter(None, next(rows)))
    for row in rows:
        if not any(row):
            return
        yield dict(zip(header, row))


def get_spreadsheet_data(filename) -> dict:
    """
    Uses `openpyxl.load_workbook` to process the specified file.
    Returns a dict of the spreadsheet data grouped by worksheet.
    """
    workbook = load_workbook(filename=filename, read_only=True, data_only=True)
    dados_empresa = worksheet_dict_reader(workbook["Empresa"])
    dados_funcionarios = worksheet_dict_reader(workbook["Funcionários"])
    dados_pagamentos = worksheet_dict_reader(workbook["Pagamentos"])

    return {
        "Empresa": list(dados_empresa)[0],
        "Funcionários": list(dados_funcionarios),
        "Pagamentos": list(dados_pagamentos),
    }



# pylint: disable=unsubscriptable-object
from typing import List, Optional

from pydantic import BaseModel

from aurorae.cnab240.v10_7.types import CNABDate, RegistrationType, RegistrationTypeEnum


class BaseConfig:
    allow_population_by_field_name = True

    @classmethod
    def alias_generator(cls, field: str) -> str:
        if field not in cls._mapping:
            raise Exception()
        return cls._mapping[field]


class SpreadsheetCompany(BaseModel):
    company_name: str
    registration_type: RegistrationType = RegistrationTypeEnum.cnpj
    registration_number: str
    bank_name: str
    bank_code: str
    bank_agency: str
    bank_agency_digit: Optional[str]
    bank_account_number: str
    bank_account_digit: Optional[str]
    bank_account_agency_digit: Optional[str]
    address_location: str
    address_number: str
    address_complement: str
    address_cep: str
    address_cep_complement: str
    address_city: str
    address_state: str

    class Config(BaseConfig):
        _mapping = {
            "company_name": "* Nome da Empresa",
            "registration_type": "* Tipo de Inscrição da Empresa",
            "registration_number": "* Número de Inscrição da Empresa",
            "bank_name": "* Nome do Banco",
            "bank_code": "* Código do Convênio no Banco",
            "bank_agency": "* Agência Mantenedora da Conta ",
            "bank_agency_digit": "* Dígito Verificador da Agência",
            "bank_account_number": "* Número da Conta Corrente",
            "bank_account_digit": "* Dígito Verificador da Conta",
            "bank_account_agency_digit": "* Dígito Verificador da Ag/Conta",
            "address_location": "Logradouro (Nome da Rua, Av, Pça, Etc)",
            "address_number": "Número (Número do Local)",
            "address_complement": "Complemento (Casa, Apto, Sala, Etc)",
            "address_cep": "CEP",
            "address_cep_complement": "Complemento do CEP",
            "address_city": "Nome da Cidade",
            "address_state": "Sigla do Estado",
        }


class SpreadsheetEmployee(BaseModel):
    name: str
    registration_type: RegistrationType = RegistrationTypeEnum.cpf
    registration_number: str
    bank_code: str
    bank_agency: str
    bank_agency_digit: Optional[str]
    bank_account_number: str
    bank_account_digit: Optional[str]
    bank_account_agency_digit: Optional[str]
    address_location: str
    address_number: str
    address_complement: str
    address_district: str
    address_cep: str
    address_cep_complement: str
    address_city: str
    address_state: str

    class Config(BaseConfig):
        _mapping = {
            "name": "Nome do Favorecido",
            "registration_type": "* Tipo de Inscrição do Favorecido",
            "registration_number": "* Nº de Inscrição do Favorecido",
            "bank_code": "Código do Banco do Favorecido",
            "bank_agency": "* Ag. Mantenedora da Cta do Favor.",
            "bank_agency_digit": "* Dígito Verificador da Agência",
            "bank_account_number": "* Número da Conta Corrente",
            "bank_account_digit": "* Dígito Verificador da Conta",
            "bank_account_agency_digit": "* Dígito Verificador da AG/Conta",
            "address_location": "Logradouro (Nome da Rua, Av, Pça, Etc)",
            "address_number": "Número (Nº do Local)",
            "address_complement": "Complemento (Casa, Apto, Etc)",
            "address_district": "Bairro",
            "address_cep": "CEP",
            "address_cep_complement": "Complemento do CEP",
            "address_city": "Nome da Cidade",
            "address_state": "Sigla do Estado",
        }


class SpreadsheetPayment(BaseModel):
    employee_name: str
    identification_number: Optional[str]
    payment_amount: str
    payment_date: CNABDate

    class Config(BaseConfig):
        _mapping = {
            "employee_name": "Funcionário",
            "identification_number": "Nº do Docum. Atribuído p/ Empresa",
            "payment_amount": "Valor do Pagamento",
            "payment_date": "Data do Pagamento",
        }


class Spreadsheet(BaseModel):
    company: SpreadsheetCompany
    employees: List[SpreadsheetEmployee]
    payments: List[SpreadsheetPayment]

    class Config(BaseConfig):
        _mapping = {
            "company": "Empresa",
            "employees": "Funcionários",
            "payments": "Pagamentos",
        }



import argparse
import os
from datetime import datetime
from pathlib import Path


def parse_args(arguments=None):
    created_at = datetime.now().isoformat()

    parser = argparse.ArgumentParser()
    parser.add_argument("input_filename", type=Path, help="input filename")
    parser.add_argument(
        "--output_filename",
        dest="output_filename",
        type=Path,
        default=f"generated_files/cnab240-{created_at}.txt",
        help="output filename",
    )
    args = parser.parse_args(arguments)

    if not os.path.exists(args.input_filename):
        parser.error("The provided entry file does not exist")

    if Path(args.output_filename).suffix != ".txt":
        parser.error("Please provide a txt file as the output")

    return args
